import os
import time

import parsel
import PySimpleGUI as sg
import requests
from termcolor import colored


def get_selector(url):
    resp = requests.get(url)
    resp.encoding = resp.apparent_encoding
    return parsel.Selector(resp.text)

# 获取书名


def get_book_url(searchkey='斗罗大陆'):
    data = {'searchkey': searchkey}
    resp = requests.post(
        'http://www.xbiquge.la/modules/article/waps.php', data=data)
    resp.encoding = resp.apparent_encoding
    selector = parsel.Selector(resp.text)
    trs = selector.css('.grid tr')
    book_info = {}
    for tr in trs[1:]:
        tds = tr.css('td')
        book_href = tds[0].css('a::attr(href)').getall()[0]
        book_name = tds[0].css('a::text').getall()[0]
        book_info[book_name] = book_href
    return book_info  # 包含书名和网址的字典

# 获取章节


def get_book_category(book_href):
    selector = get_selector(book_href)
    dds = selector.css('#list dd')
    book_category = {}
    for dd in dds:
        href = dd.css('a::attr(href)').getall()[0]
        chapter = dd.css('a::text').getall()[0]
        book_category[chapter] = 'http://www.paoshuzw.com'+href
    return book_category

# 获取章节具体内容


def get_book_content(chapter_url):
    selector = get_selector(chapter_url)
    contents = selector.css('#content::text').getall()
    return contents


# 线程锁，防止资源拥挤
#lock = threading.Lock()

# 保存函数


def save(book, charpter, contents, path):
    #global lock
    print(colored(f'Saving {book}--{charpter} ...', 'cyan'))
    # time.sleep(0.1)
    for i in contents:
        # lock.acquire()
        try:
            with open(f'{path}/{book}/{charpter}.txt', 'a', encoding='utf8') as f:
                f.write(i)
        except:
            print(f'\033[0;30;40m{i} erro!!!')
        # lock.release()

# 主函数


def main(info):
    charpter_url = info['charpter_url']
    book_name = info['book_name']
    charpter = info['charpter']
    path = info['path']
    contents = get_book_content(charpter_url)
    save(book_name, charpter, contents, path)

# 用户体验


def user(name):
    book_info = get_book_url(name)
    if book_info == {}:
        print(colored('输入的书名无效！', 'yellow'))
        # download_novel(name,path)
        raise ValueError('输入的书名无效！')
    num = 1
    print('-'*50)
    dict = {}
    for book_name, book_href in book_info.items():
        print(colored(f'{num} {book_name} {book_href}', 'green'))
        dict[str(num)] = (book_name, book_href)
        num += 1
    while 1:
        number = input('请输入序号：')
        try:
            book = dict[number]
            return book
        except KeyError:
            print(colored('Please input a existed number!', 'yellow'))


def download_novel(name, path):
    book = user(name)
    book_name = book[0]
    book_href = book[1]
    try:
        os.mkdir(path+'/'+book_name)
    except:
        pass
    book_category = get_book_category(book_href)
    for charpter, charpter_url in book_category.items():
        info = {}
        info['charpter_url'] = charpter_url
        info['book_name'] = book_name
        info['charpter'] = charpter
        info['path'] = path
        main(info)


def NovelDownloadGui():
    sg.theme('BlueMono')

    layout = [[sg.Text('Book Name',)],
              [sg.Input('斗破苍穹', key='book_name')],
              [sg.Button('Download', key='download')],
              [sg.FolderBrowse('choose a folder to save data', key='path')],
              ]

    window = sg.Window('Suluoya Novel Download', layout)

    event, values = window.read()
    name = values['book_name']
    path = values['path']
    if path == '':
        print(colored('Please choose a folder to save novels!', 'magenta'))
        window.close()
        NovelDownloadGui()
    if event == sg.WIN_CLOSED:
        window.close()
    elif event == 'download':
        print(colored('save at '+path, 'blue'))
        window.close()
        download_novel(name, path)


if __name__ == '__main__':
    NovelDownloadGui()



import json
import os
import sys

import pandas as pd
import requests
from tqdm import tqdm

sys.path.append(os.path.dirname(__file__) + os.sep + '../')
try:
    from ..log.SlyLog import slog, sprint
    from .GetData import StockData
except:
    from log.SlyLog import slog, sprint

    from GetData import StockData


class FinancialStatements(object):
    """get financial statement
    Args:
        names (list, optional): stock list. Defaults to ['贵州茅台', '隆基股份'].
        start_year (int, optional): start year. Defaults to 2018.
        start_quater (int, optional): start quater. Defaults to 1.
        end_year (int, optional): end year. Defaults to 2019.
        end_quater (int, optional): end quater. Defaults to 4.
    """

    def __init__(self, names=['贵州茅台', '隆基股份'],
                 start_year=2018, start_quater=1,
                 end_year=2019, end_quater=4):
        self.sprint = sprint()
        if type(names) == str:
            self.names = [names]
        elif type(names) == list:
            self.names = names
        self.stock_pair = StockData(names=self.names).stock_pair
        self.main_url = 'http://f10.eastmoney.com/NewFinanceAnalysis/'
        self.profit_statement_url = self.main_url+'lrbAjaxNew?'
        self.cash_flow_url = self.main_url+'xjllbAjaxNew?'
        self.balance_url = self.main_url+'zcfzbAjaxNew?'
        self.url_mode = {
            '现金流量表': self.cash_flow_url,
            '资产负债表': self.balance_url,
            '利润表': self.profit_statement_url
        }
        self.headers = {
            "Host": "f10.eastmoney.com",
            "Pragma": "no-cache",
            "Referer": "http://f10.eastmoney.com/NewFinanceAnalysis",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.114 Safari/537.36 Edg/89.0.774.76",
            "X-Requested-With": "XMLHttpRequest"
        }
        date = {1: '-03-31', 2: '-06-30', 3: '-09-30', 4: '-12-31'}
        Range = []
        if end_year-start_year >= 2:
            for i in range(start_quater, 5):
                Range.append(f'{start_year}{date[i]}')
            for i in range(start_year+1, end_year):
                for j in range(1, 5):
                    Range.append(f'{i}{date[j]}')
            for i in range(1, end_quater+1):
                Range.append(f'{end_year}{date[i]}')
        elif end_year == start_year:
            for i in range(start_quater, end_quater+1):
                Range.append(f'{end_year}{date[i]}')
        else:
            for i in range(start_quater, 5):
                Range.append(f'{start_year}{date[i]}')
            for i in range(1, end_quater+1):
                Range.append(f'{end_year}{date[i]}')
        self.Range = Range

    def params_list(self):
        self.sprint.cyan('getting financial statement...')
        RangeList = [self.Range[i:i+5] for i in range(0, len(self.Range), 5)]
        params_list = []
        for name in self.stock_pair.values():
            for Range in RangeList:
                params_list.append(
                    {
                        "companyType": "4",
                        "reportDateType": "0",
                        "reportType": "1",
                        "dates": ','.join(Range),
                        "code": name[0:2].upper()+name[3:]
                    })
        return params_list[::-1]

    def request(self, url, params):
        response = requests.get(url, headers=self.headers, params=params)
        response.encoding = response.apparent_encoding
        try:
            return json.loads(response.text)
        except:
            pass

    def get_data(self, mode='利润表'):
        for params in tqdm(self.params_list()):
            data = self.request(url=self.url_mode[mode], params=params)
            try:
                for i in data['data']:
                    yield i
            except:
                pass

    def profit_data(self, mode='利润表'):
        pass
        self.profit_dict = {
            '报告日期': 'REPORT_DATE',
            '报告形式': 'REPORT_TYPE',  # 一季报,中报,三季报,年报
            '营业总收入': 'TOTAL_OPERATE_INCOME',
            '营业收入': 'OPERATE_INCOME',
            '利息收入': 'INTEREST_INCOME',
            '实收保险费': 'EARNED_PREMIUM',
            '手续费及佣金收入': 'FEE_COMMISSION_INCOME',
            '其他营业收入': 'OTHER_BUSINESS_INCOME',
            '营业总成本': 'TOTAL_OPERATE_COST',
            '营业成本': 'OPERATE_COST',
            '研发费用': 'RESEARCH_EXPENSE',
            '营业税金及附加': 'OPERATE_TAX_ADD',
            '销售费用': 'SALE_EXPENSE',
            '管理费用': 'MANAGE_EXPENSE',
            '财务费用': 'FINANCE_EXPENSE',
            '财务费用中利息费用': 'FE_INTEREST_EXPENSE',
            '财务费用中利息收入': 'FE_INTEREST_INCOME',
            '资产减值损失': 'ASSET_IMPAIRMENT_LOSS',
            '信用减值损失': 'CREDIT_IMPAIRMENT_LOSS',
            '其他经营收益': 'TOC_OTHER',
            '公允价值变动收益': 'FAIRVALUE_CHANGE_INCOME',
            '投资收益': 'INVEST_INCOME',
            '对联营企业和合营企业的投资收益': 'INVEST_JOINT_INCOME',
            '净投资收入': 'NET_EXPOSURE_INCOME',
            '汇兑收益': 'EXCHANGE_INCOME',
            '资产处置损益': 'ASSET_DISPOSAL_INCOME',
            '资产减值收入': 'ASSET_IMPAIRMENT_INCOME',
            '其他经营收益中信用减值损失': 'CREDIT_IMPAIRMENT_INCOME',
            '其他收益': 'OTHER_INCOME',
            '营业利润': 'OPERATE_PROFIT',
            '营业外收入': 'NONBUSINESS_INCOME',
            '营业外支出': 'NONBUSINESS_EXPENSE',
            '利润总额': 'TOTAL_PROFIT',
            '所得税': 'INCOME_TAX',
            '净利润': 'NETPROFIT',
            '持续经营净利润': 'CONTINUED_NETPROFIT',
            '非持续经营净利润': 'DISCONTINUED_NETPROFIT',
            '归属于母公司股东的净利润': 'PARENT_NETPROFIT',
            '少数股东权益': 'MINORITY_INTEREST',
            '扣除非经常性损益后的净利润': 'DEDUCT_PARENT_NETPROFIT',
            '其他净利润': 'NETPROFIT_OTHER',
            '基本每股收益': 'BASIC_EPS',
            '稀释每股收益': 'DILUTED_EPS',
            '其他综合收益': 'OTHER_COMPRE_INCOME',
            '归属于母公司股东的其他综合收益': 'PARENT_OCI',
            '少数股东的其他综合收益': 'MINORITY_OCI',
            '综合收益总额': 'TOTAL_COMPRE_INCOME',
            '归属于母公司股东的综合收益总额': 'PARENT_TCI',
            '归属于少数股东的综合收益总额': 'MINORITY_TCI',
            '归属于母公司股东的其他综合收益总额': 'PRECOMBINE_TCI',
            '审计意见': 'OPINION_TYPE',
        }
        #keys = tuple(self.profit_dict.values())
        # for i in list(self.get_data(mode=mode)):
        #    yield [i[key] for key in keys]

    def statement(self, mode='利润表'):
        """
        Args:
            mode (str, optional): '利润表' or '现金流量表' or '资产负债表'. Defaults to '利润表'.

        Returns:
            [type]: dataframe
        """
        result = []
        flag = False
        columns = False
        for i in self.get_data(mode=mode):
            if not flag:
                columns = i.keys()
                flag = True
            result.append(i)
        if not columns:
            raise ValueError('No data!\nPlease check your date or stock list!')
        else:
            df = pd.DataFrame(result, columns=columns)
            return df


if __name__ == '__main__':
    fc = FinancialStatements(names=['贵州茅台', '隆基股份'],
                             start_year=2000, start_quater=1,
                             end_year=2020, end_quater=4)
    fc.statement().to_csv('temp.csv', encoding='utf8', index=False)



import pandas as pd
import numpy as np
# pip install chart_studio
# http://mtw.so/65fCUd


class app_charts(object):

    def __init__(self, theme='white'):
        '''theme:solor, pearl, white
        '''
        import cufflinks as cf
        self.cf = cf
        self.cf.set_config_file(offline=True)
        self.theme = theme

    def bar(self, df=pd.DataFrame(np.random.rand(12, 4), columns=['a', 'b', 'c', 'd']),
            mode='stack',
            sort=False):
        '''mode --> group/stack/overlay
           sort --> TrueSort bars in descending order
        '''
        df.iplot(kind='bar', barmode=mode, theme=self.theme, sortbars=sort)

    def histogram(self, df=pd.DataFrame(np.random.rand(100, 2), columns=['a', 'b'])):
        # df=self.cf.datagen.histogram(3)
        df.iplot(kind='histogram', theme=self.theme)

    def box(self, df=pd.DataFrame(np.random.rand(100, 2), columns=['a', 'b'])):
        # df=self.cf.datagen.box(20)
        df.iplot(kind='box', theme=self.theme)

    def scatter(self, df=pd.DataFrame(np.random.rand(100, 2), columns=['a', 'b']),
                data=None, xlabel='x', ylabel='y',
                mode='markers', size=5):
        '''mode:lines
                markers
                lines+markers
                lines+text
                markers+text
                lines+markers+text
            data=np.random.rand(100,2) --> try 1 try
        '''
        if data is None:
            df.iplot(kind='scatter',
                     mode=mode,
                     size=size, theme=self.theme)
        else:
            x = [i[0] for i in data]
            y = [i[1] for i in data]
            df = pd.DataFrame({xlabel: x, ylabel: y})
            df.iplot(x=xlabel, y=ylabel, xTitle=xlabel,
                     yTitle=ylabel, mode='markers')

    def line(self, df=pd.DataFrame(np.random.rand(100, 2), columns=['a', 'b']),
             subplots=False,
             share_xaxis=False):
        if subplots:
            df.iplot(subplots=True, shape=(len(df.columns), 1), shared_xaxes=share_xaxis,
                     vertical_spacing=.02, fill=True, theme=self.theme)
        else:
            df.iplot(kind='line', theme=self.theme)

    def scatter3d(self, df=pd.DataFrame({'x': np.random.rand(9),
                                         'y': np.random.rand(9),
                                         'z': np.random.rand(9),
                                         'text': range(1, 10),
                                         'categories': ['c1', 'c2', 'c3', 'c1', 'c2', 'c3', 'c1', 'c2', 'c3']
                                         })
                  ):
        columns = df.columns
        df.iplot(kind='scatter3d', x=columns[0], y=columns[1], z=columns[2],
                 text=columns[3], categories=columns[4], theme=self.theme)

    # 散点矩阵图
    def scatter_matrix(self, df=pd.DataFrame(np.random.randn(100, 3), columns=['a', 'b', 'c', ])):
        df.scatter_matrix(theme=self.theme)

    def pie(self, df=pd.DataFrame([['a', 0.2], ['b', 0.5], ['c', 0.3]], columns=['class', 'weight'])):
        columns = df.columns
        df.iplot(kind='pie', labels=columns[0],
                 values=columns[1], theme=self.theme)

    def Help(self, df=pd.DataFrame()):
        help(df.iplot())


if __name__ == '__main__':
    ac = app_charts()
    ac.scatter(data=np.random.rand(10, 2))



import json
import os
import sys

import pandas as pd
import requests
from tqdm import tqdm

sys.path.append(os.path.dirname(__file__) + os.sep + '../')
try:
    from ..log.SlyLog import slog, sprint
    from .GetData import StockData
except:
    from log.SlyLog import slog, sprint

    from GetData import StockData


class CompanyInfo(object):

    def __init__(self, names=['贵州茅台', '隆基股份']):
        self.names = names
        self.codes = list(StockData(names=self.names).stock_pair.values())
        self.sprint = sprint()
        self.main_url = 'http://f10.eastmoney.com/CompanySurvey/CompanySurveyAjax'
        self.headers = {
            "Host": "f10.eastmoney.com",
            "Referer": "http://f10.eastmoney.com/CompanySurvey/Index?type=web&code=SZ300059",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.128 Safari/537.36 Edg/89.0.774.77",
            "X-Requested-With": "XMLHttpRequest"
        }

    def params_list(self, codes):
        for code in codes:
            yield {'code': code[0:2].upper()+code[3:]}

    def request(self, params):
        response = requests.get(
            self.main_url, headers=self.headers, params=params, timeout=5)
        response.encoding = response.apparent_encoding
        return json.loads(response.text)

    def get_data(self,):
        for params in self.params_list(codes=self.codes):
            yield self.request(params=params)

    def info(self):
        info = []
        self.sprint.blue('getting company information...')
        for json_data in self.get_data():
            info.append(
                (list(json_data['jbzl'].values())+list(json_data['fxxg'].values())))
        columns = ['公司名称', '英文名称', '曾用名', 'A股代码', 'A股简称', 'B股代码', 'B股简称', 'H股代码', 'H股简称', '证券类别', '所属行业', '上市交易所', '所属证监会行业', '总经理', '法人代表', '董秘', '董事长', '证券事务代表', '独立董事', '联系电话', '电子信箱', '传真', '公司网址', '办公地址', '注册地址', '区域', '邮政编码',
                   '注册资本(元)', '工商登记', '雇员人数', '管理人员人数', '律师事务所', '会计师事务所', '公司简介', '经营范围', '成立日期', '上市日期', '发行市盈率(倍)', '网上发行日期', '发行方式', '每股面值(元)', '发行量(股)', '每股发行价(元)', '发行费用(元)', '发行总市值(元)', '募集资金净额(元)', '首日开盘价(元)', '首日收盘价(元)', '首日换手率', '首日最高价(元)', '网下配售中签率', '定价中签率']
        return pd.DataFrame(info, columns=columns)


class IndustryAnalysis(object):

    def __init__(self, names=['贵州茅台', '隆基股份']):
        self.names = names
        self.codes = list(StockData(names=self.names).stock_pair.values())
        self.sprint = sprint()
        self.url = 'http://f10.eastmoney.com/IndustryAnalysis/IndustryAnalysisAjax?'
        self.headers = {
            "Host": "f10.eastmoney.com",
            "Referer": "http://f10.eastmoney.com/IndustryAnalysis/Index?type=web&code=SZ300059",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.128 Safari/537.36 Edg/89.0.774.77",
            "X-Requested-With": "XMLHttpRequest"
        }

    def params_list(self, codes):
        params_list = []
        for code in codes:
            params_list.append({"code": code[0:2].upper()+code[3:],
                                "icode": "447"})
        return params_list

    def request(self, params):
        response = requests.get(
            self.url, headers=self.headers, params=params, timeout=5)
        response.encoding = response.apparent_encoding
        return json.loads(response.text)

    def get_data(self):
        data_list = []
        for params in tqdm(self.params_list(codes=self.codes)):
            data_list.append(self.request(params=params))
        return data_list

    @property
    def info(self):
        self.sprint.blue('getting industry analysis data...')
        industry_info = []
        growth_info = []
        valuation_info = []
        dupont_info = []
        market_size = []
        i = 0
        for data in self.get_data():
            industry_info.append({self.names[i]: data['hyzx']})  # 行业资讯
            growth_info.append(data['czxbj']['data'])  # 成长性比较
            valuation_info.append({self.names[i]: data['gzbj']['data']})  # 估值
            dupont_info.append({self.names[i]: data['dbfxbj']['data']})  # 杜邦
            market_size.append(
                {self.names[i]+'——'+'按总市值排名': data['gsgmzsz']})  # 总市值
            market_size.append(
                {self.names[i]+'——'+'按流通市值排名': data['gsgmltsz']})  # 流通市值
            market_size.append(
                {self.names[i]+'——'+'按营业收入排名': data['gsgmyysr']})  # 营业收入
            market_size.append(
                {self.names[i]+'——'+'按净利润排名': data['gsgmjlr']})  # 净利润
            i += 1
        return {
            'industry_info': industry_info,
            'growth_info': growth_info,
            'valuation_info': valuation_info,
            'dupont_info': dupont_info,
            'market_size': market_size,
        }

    def industry_info(self,info=None):
        if info == None:
            info = self.info
        lists = []
        for i in info['industry_info']:
            for j, k in i.items():
                for l in k:
                    lists.append([j, l['date'], l['title']])
        return pd.DataFrame(lists, columns=['stock', 'date', 'advisory'])

    def growth_info(self,info=None):
        if info == None:
            info = self.info
        lists = []
        for i in info['growth_info']:
            lists.append([i[0]['jc']])
            for j in i:
                lists.append(j.values())
        T1 = ['基本每股收益增长率(%)', '营业收入增长率(%)', '净利润增长率(%)']
        T2 = ['3年复合', '19A', 'TTM', '20E', '21E', '22E']
        columns = ['排名', '代码', '简称']
        for t1 in T1:
            for t2 in T2:
                columns.append(t2+'--'+t1)
        return pd.DataFrame(lists, columns=columns)

    def valuation_info(self,info=None):
        '''
        (1)MRQ市净率=上一交易日收盘价/最新每股净资产
        (2)市现率①=总市值/现金及现金等价物净增加额
        (3)市现率②=总市值/经营活动产生的现金流量净额
        '''
        if info == None:
            info = self.info
        columns = ['排名', '代码', '简称', 'PEG']
        T = {'市盈率': ['19A', 'TTM', '20E', '21E', '22E'],
             '市销率': ['19A', 'TTM', '20E', '21E', '22E'],
             '市净率': ['19A', 'MRQ'],
             '市现率①': ['19A', 'TTM'],
             '市现率②': ['19A', 'TTM'],
             'EV/EBITDA': ['19A', 'TTM']}
        for i, j in T.items():
            for k in j:
                columns.append(k+'--'+i)
        lists = []
        for i in info['valuation_info']:
            lists.append(i.keys())
            for j in i.values():
                for k in j:
                    lists.append(k.values())
        return pd.DataFrame(lists, columns=columns)

    def dupont_info(self,info=None):
        if info == None:
            info = self.info
        T1 = ['ROE(%)', '净利率(%)', '总资产周转率(%)', '权益乘数(%)']
        T2 = ['3年平均', '17A', '18A', '19A']
        columns = ['排名', '代码', '简称']
        for t1 in T1:
            for t2 in T2:
                columns.append(t2+'--'+t1)
        lists = []
        for i in info['dupont_info']:
            lists.append(i.keys())
            for j in i.values():
                for k in j:
                    lists.append(k.values())
        return pd.DataFrame(lists, columns=columns)

    def market_size(self,info=None):
        if info == None:
            info = self.info
        columns = ['排名', '代码', '简称',
                   '总市值(元)', '流通市值(元)', '营业收入(元)', '净利润(元)', '报告期']
        lists = []
        for i in info['market_size']:
            lists.append(i.keys())
            for j in i.values():
                for k in j:
                    lists.append(k.values())
        return pd.DataFrame(lists, columns=columns)


if __name__ == '__main__':
    ia = IndustryAnalysis()
    df = ia.market_size()



import os
import sys

import pandas as pd
from tqdm import tqdm

sys.path.append(os.path.dirname(__file__) + os.sep + '../')
try:
    from ..log.SlyLog import sprint
except:
    from log.SlyLog import sprint


class Macroeconomic(object):

    def __init__(self):
        import gopup as gp
        self.gp = gp
        self.g = self.gp.pro_api(token='4438684c9995613c2637de7c3264c455')
        self.sprint = sprint()

    def exchange_rate(self, currency_list=['美元', '欧元'],
                      start_date='20201010', end_date='20201015'):
        '''
        汇率
        美元、欧元、日元、英镑、卢布、韩元、澳元、加元、泰铢、港币、台币、新币
        '''
        date_range = pd.date_range(start_date, end_date, freq='1D')
        lists = []
        self.sprint.blue('getting exchange rate data...')
        for date in tqdm(date_range):
            for currency in currency_list:
                df = self.g.exchange_rate(
                    date=str(date.date()), currency=currency)
                lists.append(df)
        return pd.concat(lists)

    @property
    def get_rrr(self):
        '''
        存款准备金率
        公布时间
        生效时间
        大型金融机构 调整前
        大型金融机构 调整后
        大型金融机构 调整幅度
        中小型金融机构 调整前
        中小型金融机构 调整后
        中小型金融机构 调整幅度
        消息公布次日指数涨跌 上证
        消息公布次日指数涨跌 深证
        备注
        '''
        df = self.gp.get_rrr()
        return df


if __name__ == '__main__':
    mc = Macroeconomic()
    mc.get_rrr.to_excel('rate.xlsx',encoding='utf8')



__name__ = "Suluoya"
__author__ = 'Suluoya'
__all__ = ['Markovitz', 'GetGoodStock', 'GetData', 'GetDate', 'FinancialStatement', 'Company', 'gui']

from .GetGoodStock import GetGoodStock
from .GetDate import GetDate
from .GetData import StockData, HolidayStockData, ConstituentStock, StockAbility
from .Markovitz import Markovitz, calculate
from .gui import gui, StockGui
from .FinancialStatement import FinancialStatements
from .Company import CompanyInfo, IndustryAnalysis
import pretty_errors



import datetime
import os
import random
import sys
import time

import numpy as np
import pandas as pd
import streamlit as st
from tqdm import tqdm, trange

sys.path.append(os.path.dirname(__file__) + os.sep + '../')
try:
    from .GetData import HolidayStockData, StockData
except:
    from GetData import HolidayStockData, StockData


st.title('Suluoya Stock')


def get_stock_data():
    global StockData
    stock_list = st.text_area('stock', '隆基股份\n贵州茅台')
    stock_list = stock_list.split('\n')
    start_date = str(st.date_input(
        'start date',
        datetime.date(2019, 11, 1)))
    end_date = str(st.date_input(
        'end date',
        datetime.date(2020, 12, 31)))
    frequency = st.selectbox('frequency', ['d', 'w', 'm'])
    StockData = StockData(names=stock_list,
                          start_date=start_date,
                          end_date=end_date,
                          frequency=frequency)
    stock_pair, stock_data = StockData.stock_data[1:]
    StockData.quit()
    for i, j in stock_pair.items():
        i, ":", j
    stock_data
    data = []
    for date in stock_data['date']:
        close = stock_data[stock_data['date'] == date]['close']
        data.append(list(close))
    st.line_chart(pd.DataFrame(data, columns=stock_list))


get_stock_data()



import os
import random
import sys
import time

import numpy as np
import pandas as pd
import pretty_errors
import scipy.optimize as sco
from tqdm import tqdm, trange

sys.path.append(os.path.dirname(__file__) + os.sep + '../')
try:
    from ..chart.html_charts import html_charts
    from ..chart.svg_charts import svg_charts
    from ..log.SlyLog import slog, sprint
    from .GetData import HolidayStockData, StockData
except:
    from chart.html_charts import html_charts
    from log.SlyLog import slog, sprint
    from GetData import HolidayStockData, StockData


class calculate(object):
    def __init__(self):
        pass

    def correlation(self, x, y):
        if type(x) == list:
            x = np.array(x)
        if type(y) == list:
            y = np.array(y)
        return (np.dot([xi - np.mean(x) for xi in x], [yi - np.mean(y) for yi in y])/(len(x)-1))/((x.std())*(y.std()))

    def weight(self, lists=['A', 'B', 'C'], mode='dict'):
        if mode == 'dict':
            weights = np.random.dirichlet(np.ones(len(lists)), size=1)[0]
            rand = {}
            for i in range(0, len(lists)):
                rand[lists[i]] = weights[i]
            return rand
        elif mode == 'list':
            weights = np.random.dirichlet(np.ones(len(lists)), size=1)[0]
            rand = {}
            for i in range(0, len(lists)):
                rand[lists[i]] = weights[i]
            return list(rand.values())

    def equation(self, xy=[[1, 2], [2, 3]]):
        from sympy import Symbol, solve
        a = Symbol('a')
        b = Symbol('b')
        f1 = (xy[0][0]**2)/(a**2)-(xy[0][1]**2)/(b**2)-1
        f2 = (xy[1][0]**2)/(a**2)-(xy[1][1]**2)/(b**2)-1
        s = solve([f1, f2], [a, b])
        return s[0][0]**2, s[0][1]**2


class Markovitz(calculate):
    """Markovitz Portfolio
    Args:
        names (list, optional): stock list. Defaults to ['贵州茅台', '隆基股份'].
        start_date (str, optional): start date. Defaults to '2019-01-01'.
        end_date (str, optional): end date. Defaults to '2020-01-01'.
        frequency (str, optional): frequency. Defaults to "w".
        holiday (bool, optional): holiday mode. Defaults to False.
        holiday_name (str, optional): holiday name. Defaults to '国庆节'.
        before (int, optional): how many days before the holidays. Defaults to -21.
        after (int, optional): how many days after the holidays. Defaults to 21.
        no_risk_rate (float, optional): risk-off interest rate. Defaults to 0.0185.
    """

    def __init__(self,
                 names=['贵州茅台', '隆基股份'],
                 start_date='2019-01-01',
                 end_date='2020-01-01',
                 frequency="w",
                 holiday=False,
                 holiday_name='国庆节',
                 before=-21, after=21,
                 no_risk_rate=0.0185
                 ):

        self.html_charts = html_charts()
        self.sprint = sprint()
        self.slog = slog('Markovitz ')
        self.frequency = frequency
        if frequency == 'd':
            self.no_risk_rate = no_risk_rate / 365
        elif frequency == 'w':
            self.no_risk_rate = no_risk_rate / 52
        elif frequency == 'm':
            self.no_risk_rate = no_risk_rate / 12
        else:
            raise ValueError('frequency should be "d" or "w" or "m"!')
        self.names = names
        self.start_date = start_date
        self.end_date = end_date
        self.slog.log(f'time: {self.start_date} ~~ {self.end_date}', mode=1)
        self.slog.log(f'stocks: {self.names}', mode=1)
        self.slog.log(f'frequency: {self.frequency}', mode=1)
        self.slog.log(
            f'risk-free interest rate: {round(self.no_risk_rate,6)}', mode=1)
        try:
            os.makedirs('./picture')
        except:
            pass
        cache = False
        while cache == False:
            try:
                self.stock_data = pd.read_csv(f'cache\\{str(self.names)}.csv')
                self.sprint.cyan('Use cache data to calculate!')
                self.codes = None
                self.stock_pair = None
                break
            except:
                self.sprint.cyan('No cache data found!')
                cache = True
                self.sprint.red(
                    'Please make sure that all of the stocks are in the market!')
                self.sprint.green('initializing...')
                if holiday == False:
                    self.StockData = StockData(names=self.names,
                                               start_date=self.start_date,
                                               end_date=self.end_date,
                                               frequency=self.frequency,
                                               cache=cache)
                    try:
                        self.codes, self.stock_pair, self.stock_data = self.StockData.stock_data
                        self.StockData.quit()
                    except:
                        raise ValueError('data get error, please retry!')
                else:
                    if holiday_name == None:
                        holiday_name = '国庆节'
                    self.holiday_name = holiday_name
                    start_date = self.start_date.replace('-', '')
                    end_date = self.end_date.replace('-', '')
                    self.HolidayStockData = HolidayStockData(names=names,
                                                             start_date=start_date,
                                                             end_date=end_date,
                                                             holiday=holiday_name,
                                                             frequency=frequency,
                                                             before=before, after=after,
                                                             cache=cache)
                    try:
                        self.codes, self.stock_pair, self.stock_data = self.HolidayStockData.HolidayNearbyData
                    except:
                        raise ValueError('data get error, please retry!')
                date = self.stock_data[['date']
                                       ].drop_duplicates().values.tolist()
                for i in self.names:
                    k_data = self.stock_data[self.stock_data['name'] == i][[
                        'open', 'close', 'low', 'high']].values.tolist()
                    kline = self.html_charts.kline(x=date, data=k_data)
                    self.html_charts.save(kline, path=f"picture\\{i}-kline")
                data = {}
                for i in self.names:
                    data[i] = list(
                        self.stock_data[self.stock_data['name'] == i]['pctChg'])
                line = self.html_charts.line(x=date, y=data)
                self.html_charts.save(line, path="picture\\pctChg")
                break

    def sharp(self, weights=[], stock_list=[]):
        if weights == []:
            weights = len(self.names) * [1. / len(self.names), ]
        if stock_list == []:
            stock_list = self.names
        df = self.stock_data[['pctChg', 'name']]
        covs = []
        for i in range(0, len(stock_list)):
            for j in stock_list[i:]:
                if stock_list[i] != j:
                    covs.append([stock_list[i], j])
        for i in covs:
            x = list(df[df['name'] == i[0]]['pctChg'])
            y = list(df[df['name'] == i[1]]['pctChg'])
            i.append(self.correlation(x, y))
        dic = {}
        rate = 0
        risk = 0
        c = 0
        rand = {}
        for i in stock_list:
            weight = weights[c]
            rand[i] = weight
            data = list(df[df['name'] == i]['pctChg'])
            Mean = np.mean(data)
            Std = np.std(data)
            rate += Mean*weight
            risk += (weight*Std)**2
            dic[i] = weight*Std
            c += 1
        for i in covs:
            risk += 2*i[2]*dic[i[0]]*dic[i[1]]
        sharp = (rate-self.no_risk_rate)/risk
        return {'sharp': sharp, 'rate': rate, 'risk': risk, 'weight': rand}

    def max_sharp(self, weights, *args):
        stock_list = args[0]
        self.slog.log(f'weights: {weights}', mode=1)
        return -self.sharp(weights=weights, stock_list=stock_list)['sharp']

    def portfolio(self, stock_list=[], accurate=True, number=500):
        """[summary]

        Args:
            stock_list (list, optional): stock list. Defaults to [].
            accurate (bool, optional): Calculate Mode, by iteration algorithm(True) or generating scatters(False). Defaults to True.
            number (int, optional): number of scatters. Defaults to 500.

        Returns:
            [type]: result(dataframe or list)
        """
        if stock_list == []:
            stock_list = self.names
        self.sprint.magenta(f'building a portfolio for {stock_list}')
        try:
            os.makedirs('./result')
        except:
            pass
        if accurate == True:
            opts = sco.minimize(fun=self.max_sharp,
                                x0=len(stock_list) * [1. / len(stock_list), ],
                                method='SLSQP',
                                args=(stock_list,),
                                bounds=tuple((0, 1)
                                             for x in range(len(stock_list))),
                                constraints={'type': 'eq',
                                             'fun': lambda x: np.sum(x) - 1}
                                )
            count = 0
            result = {}
            ans = list(opts['x'])
            for i in stock_list:
                result[i] = ans[count]
                count += 1
            pie = self.html_charts.pie(weights=result)
            self.html_charts.save(pie, path='picture\\weights-pie')
            result = [result, -opts['fun']]
            self.slog.log(f'weights:{result[0]}\nsharp:{result[1]}')
            df = pd.DataFrame([result], columns=['weights', 'sharp'])
            df.to_csv('result\\accurate_result.csv',
                      index=False, encoding='utf8')
            df.to_excel('result\\accurate_result.xlsx',
                        index=False, encoding='utf8')
            return result
        else:
            lists = []
            for i in trange(0, number):
                weights = self.weight(lists=stock_list, mode='list')
                sharp = self.sharp(
                    weights=weights, stock_list=stock_list)
                lists.append(list(sharp.values()))
            df = pd.DataFrame(
                lists, columns=['sharp', 'rate', 'risk', 'weight'])
            df = df.sort_values('sharp', ascending=False)
            risk = []
            rate = {'sharp': []}
            for j in df.itertuples():
                risk.append(getattr(j, 'risk'))
                rate['sharp'].append(getattr(j, 'rate'))
            scatter = self.html_charts.scatter(x=risk, y=rate)
            self.html_charts.save(
                scatter, path='picture\\sharp-scatter(还是自己画吧QAQ...)')
            weights = list(df['weight'])[0]
            sharp = list(df['sharp'])[0]
            pie = self.html_charts.pie(weights=weights)
            self.html_charts.save(pie, path='picture\\weights-pie')
            self.slog.log(f'weights:{weights}\nsharp:{sharp}')
            sharp_max = list(df.iloc[0][['rate', 'risk']])
            sharp_min = list(df.iloc[-1][['rate', 'risk']])
            e = self.equation([sharp_max, sharp_min])
            self.slog.log(f'hyperbolic equation\na2={e[0]}, b2={e[1]}')
            df.to_excel('result\\not_accurate_result.xlsx',
                        index=False, encoding='utf8')
            df.to_csv('result\\not_accurate_result.csv',
                      index=False, encoding='utf8')
            return df


if __name__ == '__main__':
    Markovitz = Markovitz(names=['隆基股份', '五粮液', '贵州茅台'],
                          start_date='2019-12-01',
                          end_date='2020-12-31',
                          frequency='w',
                          holiday=False,
                          )
    print(Markovitz.portfolio(accurate=False,))
